--[[---------------------------------------------------------------------------------
  XAD by Thirsterhall
  
  TODO:
  
----------------------------------------------------------------------------------]]

--[[---------------------------------------------------------------------------------
  Create the AddOn object and create a local binding for AceLocale
----------------------------------------------------------------------------------]]
XAD = AceLibrary("AceAddon-2.0"):new(
	"AceEvent-2.0", 
	"AceConsole-2.0", 
	"AceDB-2.0",    
	"AceDebug-2.0"
--	"AceHook-2.0"
)
XAD:SetDebugLevel(2)

XAD:RegisterDB("XADDB")
XAD:RegisterDefaults('profile', {
    position = { x=0, y=0 },
    scale = 1,
    locked = true
})

-- Load up the libraries
local L = AceLibrary:GetInstance("AceLocale-2.0"):new("XAD")
local BS = AceLibrary:GetInstance("Babble-Spell-2.0")
local BZ = AceLibrary:GetInstance("Babble-Zone-2.0")
local SC = AceLibrary:GetInstance("SpellCache-1.0")
local metro = AceLibrary("Metrognome-2.0")
local PT = PeriodicTableEmbed:GetInstance("1")

--[[---------------------------------------------------------------------------------
  This is the actual addon object
----------------------------------------------------------------------------------]]

function XAD:OnInitialize()
	--tabel to hold the buttons
	self.Buttons = {}
	local Options = {
		type="group",
		args = {
			lock = {
				name = "Lock", type = "toggle",
				desc = "Lock the frame",
				get = function() return self.db.profile.locked end,
				set = function(v) self:toggleMove(v) end,
			},
			scale = {
				name = "Scale",
			    type = 'range',
				desc = "Scale the bars",
				get = function() return self.db.profile.scale end,
				set = function(v) self:scale(v) end,
				min = .1,
				max = 2, 
				step = 0.05, -- every 5%
				isPercent = true,
			},
	}}
	self:RegisterChatCommand({ "/xad"}, Options)
end

function XAD:OnEnable()
	-- set up a few constants
	-- constants set in OnEnable so OnDisable and clear them out to save a bit more mem when on standbay
	self.Constants = {
		questionmark =  "Interface\\Icons\\INV_Misc_QuestionMark",
		NORMALTEX_RATIO = 1.833333333,
		HealthStoneSpells = {
			'Create Healthstone (Major)',
			'Create Healthstone (Greater)',
			'Create Healthstone',
			'Create Healthstone (Lesser)',
			'Create Healthstone (Minor)',
		},
		ManaStoneSpells = {
			"Conjure Mana Ruby",
			"Conjure Mana Citrine",
			"Conjure Mana Jade",
			"Conjure Mana Agate",
		},
		ConjureFoodSpells = {
			"Conjure Food"
		},
		ConjureWaterSpells = {
			"Conjure Water"
		},
	}

	-- create the  buttons
	self.Buttons = self:CreateButtons()
	--
	--setup delay timer for bag updates
	metro:Register("XAD_Refresh", self.OnTick, 1, self)
	metro:Register("XAD_Cooldown1", self.OnCDTick, 1, self, 1)
	metro:Register("XAD_Cooldown2", self.OnCDTick, 1, self, 2)
	metro:Register("XAD_Cooldown3", self.OnCDTick, 1, self, 3)
	metro:Register("XAD_Cooldown4", self.OnCDTick, 1, self, 4)
	metro:Register("XAD_Cooldown5", self.OnCDTick, 1, self, 5)
	metro:Register("XAD_CooldownHS", self.OnCDTick, 1, self, 'HS')
	metro:Register("XAD_CooldownMS", self.OnCDTick, 1, self, 'MS')
	self:PositionButtons()
	self:AddCustomPTSets()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_LEAVING_WORLD")
	
end

function XAD:OnDisable()
	self.Constants = nil
	
	--stop and get rid of the metro timers
	metro:Stop("XAD_Refresh")
	metro:UnregisterMetro("XAD_Refresh")
end

--[[---------------------------------------------------------------------------------
				Events
----------------------------------------------------------------------------------]]

function XAD:PLAYER_ENTERING_WORLD()
	self.dirty = true
	metro:Start("XAD_Refresh")
	self:RegisterEvent("BAG_UPDATE")
end


function XAD:PLAYER_LEAVING_WORLD()
	self:UnregisterEvent("BAG_UPDATE")
end

function XAD:BAG_UPDATE()
	self.dirty = true
	metro:Start("XAD_Refresh")
end


--[[---------------------------------------------------------------------------------
			Create the UI elements
----------------------------------------------------------------------------------]]
function XAD:CreateButtons()
	-- create a backdrop for the buttons
	local buttons = CreateFrame('Frame', nil, UIParent )
	-- enable the mouse and make the frame movable
	buttons:EnableMouse(true)
	buttons:SetMovable(true)

	--set the width = button width * 5 plus a bit of padding
	buttons:SetWidth(32*6 + 4)
	-- set height button height plus a bit of padding
	buttons:SetHeight(32+4)
	--setup a backdrop for when moving the buttons around
	buttons:SetBackdrop({
		bgFile = 'Interface\\Tooltips\\UI-Tooltip-Background',
		edgeFile = '',
		edgeSize = 0,
		tile = true,
		tileSize = 16,
		insets = {left = 0, right = 0, top = 0, bottom = 0}
	})
	buttons:SetBackdropColor(0, 0, 0, 0.6)
	-- setup functions for moving positioning the buttons
	buttons:SetScript("OnMouseDown",function()
		if arg1 == "LeftButton" and not XAD.db.profile.locked then
			this:StartMoving()
		end
	end)
	buttons:SetScript("OnMouseUp",function()
		if arg1 == "LeftButton" and not XAD.db.profile.locked then
			this:StopMovingOrSizing()
			XAD.db.profile.position.x = this:GetLeft() * this:GetEffectiveScale()
			XAD.db.profile.position.y = this:GetBottom() * this:GetEffectiveScale()
		end
	end)
	-- if the fame is locked then hide the background
	if(self.db.profile.locked) then
		buttons:SetBackdropColor(0, 0, 0, 0.0)
	end

	for i=1, 6 do
		buttons[i] = self:CreateButton(i,'XADButton'..i,buttons,32)
		if i>1 then buttons[i]:SetPoint('LEFT', buttons[i-1], 'RIGHT') end
	end
	buttons[1]:SetPoint('TOPLEFT',buttons,'TOPLEFT', 1, -1)
	return buttons
end

function XAD:CreateButton (id, name, parent, size)
	local button = CreateFrame("Button", name, parent)
	button:SetID(id)
	button:SetAlpha(parent:GetAlpha())
	button:SetHeight(size)
	button:SetWidth(size)
	
	--icon texture
	local iconTexture = button:CreateTexture(name .. "Icon", "BACKGROUND")
	iconTexture:SetTexCoord(0.06, 0.94, 0.06, 0.94)
	iconTexture:SetAllPoints(button)
	
	--normal, pushed, highlight, checked textures
	local normalTexture = button:CreateTexture(name .. "NormalTexture")
	normalTexture:SetTexture("Interface\\Buttons\\UI-Quickslot2")
	normalTexture:SetHeight( size)
	normalTexture:SetWidth( size)
	normalTexture:SetPoint("CENTER", button, "CENTER", 0, -1)
	
	button:SetNormalTexture(normalTexture)
	button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square")
	--button:SetCheckedTexture("Interface\\Buttons\\CheckButtonHilight")
	
	--hotkey
	local hotkey = button:CreateFontString(name .. "HotKey", "ARTWORK")
	hotkey:SetFontObject(NumberFontNormalSmallGray)
	hotkey:SetPoint("TOPRIGHT", button, "TOPRIGHT", 2, -2)
	hotkey:SetJustifyH("RIGHT")
	hotkey:SetWidth(size)
	hotkey:SetHeight(10)
	hotkey:Hide();
	
	--count
	local count = button:CreateFontString(name .. "Count", "ARTWORK")
	count:SetFontObject(NumberFontNormalSmallGray)
	count:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 2, 2)
	count:SetJustifyH("RIGHT")
	count:SetWidth(size)
	count:SetHeight(10)
	count:Hide();
	
	--cooldown model
	local cooldown = CreateFrame("Model", name .. "Cooldown", button, "CooldownFrameTemplate")
	cooldown:SetAllPoints(button)
	
	--cooldown text
	local count = button:CreateFontString(name .. "CoolDownText", "ARTWORK")
	count:SetFontObject(NumberFontNormal)
	count:SetPoint("CENTER", button, "CENTER")
	count:SetJustifyH("CENTER")
	count:SetWidth(size)
	count:SetHeight(20)
	count:Hide();
	
	--setup the button scripts and mouse buttons
	button:EnableMouse(true)
	button:SetScript('OnEnter', function()  end)
	button:SetScript('OnLeave', function()  end)
	button:RegisterForClicks('LeftButtonUp','RightButtonUp','MiddleButtonUp','Button4Up','Button5Up')
	button:SetScript('OnClick', function() XAD:OnClick(arg1) end )

	-- return the button
	return button
end

function XAD:scale(a1)
	self:LevelDebug(2, "XAD:scale(%s)",a1)
	self.db.profile.scale = a1
	self:PositionButtons()
end

function XAD:PositionButtons()
	if self.db.profile.scale then 
		self.Buttons:SetScale(self.db.profile.scale) 
		for i=1, 6 do
			self.Buttons[i]:SetScale(self.db.profile.scale) 
		end
		--might need to add scaling to each of the buttons here
	end
	if self.db.profile.position then
		self.Buttons:SetPoint('BOTTOMLEFT', UIParent, 'BOTTOMLEFT', 
			self.db.profile.position.x/self.Buttons:GetEffectiveScale(), 
			self.db.profile.position.y/self.Buttons:GetEffectiveScale())
	else
        self.Buttons:SetPoint('CENTER', UIParent, 'CENTER')
    end
end

function XAD:toggleMove(a1)
    self:LevelDebug(2, "XAD:toggleMove(%s)",a1)
	self.db.profile.locked = a1
	if a1 then
		self.Buttons:SetBackdropColor(0, 0, 0, 0.0)
	else
		self.Buttons:SetBackdropColor(0, 0, 0, 0.6)
	end
end
--[[---------------------------------------------------------------------------------
			Core addon code
----------------------------------------------------------------------------------]]

function XAD:OnTick()
	if not self.dirty then return end
	-- hold off refreshing things untill combat is over
	--if UnitAffectingCombat("player") then return end
	metro:Stop("XAD_Refresh")
	self:RefreshButtons()
	self.dirty = nil
end

function XAD:OnCDTick(id)
	self:LevelDebug(2, "XAD:OnCDTick(%s)",id)
	if type(id) == 'number' then
		local timeleft = self.Buttons[id].Timer - GetTime()
		if not self.Buttons[id].stoned then self.Buttons[id]:SetAlpha(0.5) end
		if timeleft < 1 then
			self.Buttons[id]:SetAlpha(1.0)
			self.Buttons[id].Timer = nil
			metro:Stop("XAD_Cooldown"..id)
			getglobal("XADButton"..id.."CoolDownText"):SetText("")
			getglobal("XADButton"..id.."CoolDownText"):Hide()
		elseif(timeleft > 60)then
			getglobal("XADButton"..id.."CoolDownText"):SetText((floor(timeleft / 60)) +1)
			getglobal("XADButton"..id.."CoolDownText"):Show()
		else
			getglobal("XADButton"..id.."CoolDownText"):SetText(floor(timeleft))
			getglobal("XADButton"..id.."CoolDownText"):Show()
		end
	else
		-- we have a HS or MS cooldown counter
		if id == 'HS' and self.Buttons.HSTimer then
			if self.Buttons.HSTimer - GetTime() < 1 then
				self.Buttons.HSTimer = nil
				metro:Stop('XAD_CooldownHS')
				self:RefreshButtons()
			end
		elseif id == 'MS' and self.Buttons.MSTimer then
			if self.Buttons.MSTimer - GetTime() < 1 then
				self.Buttons.MSTimer = nil
				metro:Stop('XAD_CooldownMS')
				self:RefreshButtons()
			end
		end
	end
end

function XAD:AddCustomPTSets()
	PT:RegisterCustomSet("6948","xadhearthstone")
	PT:RegisterCustomSet("5512:100 19004:110 19005:120 5511:250 19006:275 19007:300 5509:500 19008:550 19009:600 5510:800 19010:880 19011:960 9421:1200 19012:1320 19013:1440","xadhealthstone")
	PT:RegisterCustomSet("5513:600 5514:400 8007:850 8008:1100","xadmanastone")
	PT:RegisterCustomSet({"food", "foodbonus", "foodstat","foodcombohealth"},"xadfood")
	PT:RegisterCustomSet({"foodperc", "foodpercbonus", "foodcomboperc"},"xadfoofperc")
	PT:RegisterCustomSet({"water", "foodcombomana"},"xadwater")
	PT:RegisterCustomSet({"waterperc", "foodcomboperc"},"xadwaterperc")
end

function XAD:RefreshButtons()
	local btn=1
	local bag, slot, val
	local bagp, slotp, valp
	--update button 1 hearth stone
	bag, slot = PT:GetBest("xadhearthstone")
	self:UpdateButton(btn, bag, slot)
	
	--update button 2 health pots
	btn=btn+1
	bag, slot = nil, nil
	bag, slot = PT:GetBest("xadhealthstone")
	-- if we have an healthstone but it's on cooldown then ignore it
	if bag and GetContainerItemCooldown(bag,slot)>0  then 
		local _, _, s = metro:Status('XAD_CooldownHS')
		if not s then 
			metro:Start('XAD_CooldownHS')
			local st, d = GetContainerItemCooldown(bag,slot)
			self.Buttons.HSTimer = st + d
		end
		bag, slot = nil, nil 
	end
	if bag then self.Buttons[btn].stoned = true else self.Buttons[btn].stoned = nil end
	if not bag and GetRealZoneText()==BZ["Alterac Valley"] then bag, slot = PT:GetBest("potionhealalterac") end
	if not bag then bag, slot = PT:GetBest("potionhealall") end
	self:UpdateButton(btn, bag, slot)
	
	--update button 3 Mana pots
	btn=btn+1
	bag, slot = nil, nil
	bag, slot = PT:GetBest("xadmanastone")
	-- if we have an manastone but it's on cooldown then ignore it
	if bag and GetContainerItemCooldown(bag,slot)>0  then 
		local _, _, s = metro:Status('XAD_CooldownMS')
		if not s then 
			metro:Start('XAD_CooldownMS')
			local st, d = GetContainerItemCooldown(bag,slot)
			self.Buttons.HSTimer = st + d
		end
		bag, slot = nil, nil 
	end
	if bag then self.Buttons[btn].stoned = true else self.Buttons[btn].stoned = nil end
	if not bag and GetRealZoneText()==BZ["Alterac Valley"] then bag, slot = PT:GetBest("potionmanaalterac") end
	if not bag then bag, slot = PT:GetBest("potionmanaall") end
	self:UpdateButton(btn, bag, slot)
	
	--update button 4 Food
	btn=btn+1
	bag, slot = nil, nil
	bag, slot = PT:GetBest("foodbreadconjured")
	if not bag then 
		bag, slot, val = PT:GetBest("xadfood") 
		bagp, slotp, valp = PT:GetBest("xadfoofperc") 
		-- if the percent heal item restores more than the normal item use it
		if bagp then if (valp*UnitHealthMax("player")/100) > val then bag, slot = bagp, slotp end end
	end
	self:UpdateButton(btn, bag, slot)
	
	--update button 5 Water
	btn=btn+1
	bag, slot = nil, nil
	bag, slot = PT:GetBest("waterconjured")
	if not bag then 
		bag, slot, val = PT:GetBest("xadwater") 
		bagp, slotp, valp = PT:GetBest("xadwaterperc") 
		-- if the percent heal item restores more than the normal item use it
		if bagp then if (valp*UnitManaMax("player")/100) > val then bag, slot = bagp, slotp end end
	end
	self:UpdateButton(btn, bag, slot)
	
	--update button 6 Bandage
	btn=btn+1
	bag, slot = nil, nil
	if GetRealZoneText()==BZ["Warsong Gulch"] then 
		bag, slot = PT:GetBest("bandageswarsong")
	elseif GetRealZoneText()==BZ["Arathi Basin"] then 
		bag, slot = PT:GetBest("bandagesarathi")
	elseif GetRealZoneText()==BZ["Alterac Valley"] then 
		bag, slot = PT:GetBest("bandagesalterac")
	end
	if not bag then bag, slot = PT:GetBest("bandagesgeneral") end
	self:UpdateButton(btn, bag, slot)
	
	self.Buttons:Show()
end

function XAD:UpdateButton(id, bag, slot)
    self:LevelDebug(2, "XAD:UpdateButton(%s, %s, %s)",id, bag, slot)
	if bag and slot then
		self.Buttons[id]:SetNormalTexture(GetContainerItemInfo(bag,slot))
		if GetContainerItemCooldown(bag,slot)>0 then
			self:LevelDebug(2,"item on button %s has a cooldown", id)
			if not self.Buttons[id].Timer then 
				self:LevelDebug(2,"item on button %s has a cooldown", id)
				metro:Start('XAD_Cooldown'..id)
				local st, d = GetContainerItemCooldown(bag,slot)
				self.Buttons[id].Timer = st + d
			end
		end
		getglobal("XADButton"..id.."Count"):SetText(self:CountItem(bag,slot))
		getglobal("XADButton"..id.."Count"):Show()
		self.Buttons[id]:Show()
	else
		if id == 2 then
			local _, _, t  = self:GetSpellData(self.Constants.HealthStoneSpells)
			if t then 
				self.Buttons[id]:SetNormalTexture(t)
				getglobal("XADButton"..id.."Count"):SetText("--")
				return
			end
		elseif id == 3 then
			local _, _, t  = self:GetSpellData(self.Constants.ManaStoneSpells)
			if t then 
				self.Buttons[id]:SetNormalTexture(t)
				getglobal("XADButton"..id.."Count"):SetText("--")
				return
			end
		elseif id == 4 then
			local _, _, t  = self:GetSpellData(self.Constants.ConjureFoodSpells)
			if t then 
				self.Buttons[id]:SetNormalTexture(t)
				getglobal("XADButton"..id.."Count"):SetText("--")
				return
			end
		elseif id == 5 then
			local _, _, t  = self:GetSpellData(self.Constants.ConjureWaterSpells)
			if t then 
				self.Buttons[id]:SetNormalTexture(t)
				getglobal("XADButton"..id.."Count"):SetText("--")
				return
			end
		end
		self.Buttons[id]:Hide()
	end
end

function XAD:CountItem(b, s)
	local count = 0
	local itemlink = GetContainerItemLink(b, s)
	local _,_,sitemID = string.find(itemlink or "", "item:(%d+):%d+:%d+:%d+")
	sitemID = tonumber(sitemID)

--[[	for bag,slot,val in PT:BagIter({sitemID}) do
		local _, stack = GetContainerItemInfo(bag, slot)
		count = count + stack
	end
	return count
	--]]
	for bag=0,4 do
		for slot=1,GetContainerNumSlots(bag) do
			local itemlink = GetContainerItemLink(bag, slot)
			local _,_,itemID = string.find(itemlink or "", "item:(%d+):%d+:%d+:%d+")
			itemID = tonumber(itemID)
			if sitemID == itemID then
				local _, stack = GetContainerItemInfo(bag, slot)
				count = count + stack
			end
		end
	end
	return count

end

function XAD:OnClick(button)
    self:LevelDebug(2, "XAD:OnClick(%s)",button)
	if not self.db.profile.locked then return end
	local bag, slot, val
	local bagp, slotp, valp
	if button == "LeftButton" then
		-- Hearth Stone
		if this:GetID() == 1 then
			bag, slot = PT:GetBest("xadhearthstone")
			if bag and slot and GetContainerItemCooldown(bag,slot) == 0 then UseContainerItem(bag, slot) return end
		-- Health potion
		elseif this:GetID() == 2 then
			--check for healthstone first
			bag, slot = PT:GetBest("xadhealthstone")
			if bag and slot and GetContainerItemCooldown(bag,slot) == 0 then UseContainerItem(bag, slot)  return end
			bag, slot = nil, nil
			-- from pothead added check for cooldown
			local h = UnitHealthMax("player") - UnitHealth("player")
			local v = function(bag,slot,val) return val/2 <= h end
			local f = function(a1,a2,a3,b1,b2,b3) return (a3*1.125) <= h and (b3*.875) > h or (a3*1.125) <= h and a3 > b3 or a3 > h and b3 > h and a3 < b3 end
			if GetRealZoneText()==BZ["Alterac Valley"] then bag, slot, val = PT:GetBest("potionhealalterac", f, v) end
			if not bag then bag, slot, val = PT:GetBest("potionhealall", f, v) end
			if bag and slot and GetContainerItemCooldown(bag,slot) == 0  then UseContainerItem(bag, slot) return end
		-- Mana Potion
		elseif this:GetID() == 3 then
			bag, slot = PT:GetBest("xadmanastone")
			if bag and slot and GetContainerItemCooldown(bag,slot) == 0 then UseContainerItem(bag, slot)  return end
			bag, slot = nil, nil
			-- from pothead added check for cooldown
			local m = UnitManaMax("player") - UnitMana("player")
			local v = function(bag,slot,val) return val/2 <= m end
			local f = function(a1,a2,a3,b1,b2,b3) return a3 <= m and b3 > m or a3 <= m and a3 > b3 or a3 > m and b3 > m and a3 < b3 end
			if GetRealZoneText()==BZ["Alterac Valley"] then bag, slot = PT:GetBest("potionmanaalterac", f, v) end
			if not bag then bag, slot, val = PT:GetBest("potionmanaall", f, v) end
			if bag and slot and GetContainerItemCooldown(bag,slot) == 0  then UseContainerItem(bag, slot) return end
		-- Food
		elseif this:GetID() == 4 then
			local h = UnitHealthMax("player") - UnitHealth("player")
			if h == 0 then return end
			local f = function(a1,a2,a3,b1,b2,b3) return a3 <= h and b3 > h or a3 <= h and a3 > b3 or a3 > h and b3 > h and a3 < b3 end
			bag, slot = PT:GetBest("foodbreadconjured", f)
			if not bag then
				bag, slot, val = PT:GetBest("xadfood", f)
				h = UnitHealth("player")/UnitHealthMax("player") * 100
				bagp, slotp, valp = PT:GetBest("xadfoofperc") 
				-- if the percent heal for normal item less then percent item show percent item since it's better
				if (valp*UnitHealthMax("player")/100) > val then bag, slot = bagp, slotp end
			end
			if bag and slot and GetContainerItemCooldown(bag,slot) == 0  then UseContainerItem(bag, slot) return end
		-- Water
		elseif this:GetID() == 5 then
			-- from pothead added check for cooldown
			local m = UnitManaMax("player") - UnitMana("player")
			if m == 0 then return end
			local f = function(a1,a2,a3,b1,b2,b3) return a3 <= m and b3 > m or a3 <= m and a3 > b3 or a3 > m and b3 > m and a3 < b3 end
			bag, slot = PT:GetBest("waterconjured", f)
			if not bag then
				bag, slot, val = PT:GetBest("xadwater", f)
				m = UnitHealth("player")/UnitManaMax("player") * 100
				bagp, slotp, valp = PT:GetBest("xadwaterperc") 
				-- if the percent heal for normal item less then percent item show percent item since it's better
				if (valp*UnitManaMax("player")/100) > val then bag, slot = bagp, slotp end
			end
			if bag and slot and GetContainerItemCooldown(bag,slot) == 0  then UseContainerItem(bag, slot) return end
		-- Bandage
		elseif this:GetID() == 6 then
			-- from pothead added check for cooldown
			unit = unit or UnitExists("target") and UnitIsFriend("target", "player") and "target" or "player"
			local h = UnitHealthMax(unit) - UnitHealth(unit)
			local v = function(bag,slot,val) return val/2 <= h end
			local f = function(a1,a2,a3,b1,b2,b3) return a3 <= h and b3 > h or a3 <= h and a3 > b3 or a3 > h and b3 > h and a3 < b3 end
			if GetRealZoneText()==BZ["Warsong Gulch"] then 
				bag, slot = PT:GetBest("bandageswarsong", f, v)
			elseif GetRealZoneText()==BZ["Arathi Basin"] then 
				bag, slot = PT:GetBest("bandagesarathi", f, v)
			elseif GetRealZoneText()==BZ["Alterac Valley"] then 
				bag, slot = PT:GetBest("bandagesalterac", f, v)
			end
			if not bag then bag, slot, val = PT:GetBest("bandagesgeneral", f, v) end
			if bag and slot then
				UseContainerItem(bag, slot)
				if SpellIsTargeting() then SpellTargetUnit(unit) end
				return
			end
		end
	elseif button == "RightButton" then
		-- health stone
		if this:GetID() == 2 then
			local bag, slot = PT:GetBest("healthstone")
		    self:LevelDebug(2, "Health stone at bag %s, %s", bag, slot)
			if bag and slot then
				self:Trade(bag, slot)
			else
				self:LevelDebug(2, "no stone so create one")
				self:CastSpell(self.Constants.HealthStoneSpells)
			end
		--mana stone
		elseif this:GetID() == 3 then
			local bag, slot = PT:GetBest("manastone")
			if not bag then
				self:CastSpell(self.Constants.ManaStoneSpells)
			end
		--create food
		elseif this:GetID() == 4 then
			local bag, slot = PT:GetBest("foodbreadconjured")
			if bag and slot and UnitIsPlayer("target") and UnitIsFriend("player","target") then
				self:Trade(bag, slot)
			else
				self:CastSpell(self.Constants.ConjureFoodSpells)
			end
		--create water
		elseif this:GetID() == 5 then
			local bag, slot = PT:GetBest("waterconjured")
			if bag and slot and UnitIsPlayer("target") and UnitIsFriend("player","target") then
				self:Trade(bag, slot)
			else
				self:CastSpell(self.Constants.ConjureWaterSpells)
			end
		end
	end
end

function XAD:Trade(bag, slot)
	if (UnitIsPlayer("target") and (UnitIsFriend("player","target"))) then
		if (not UnitCanCooperate("player", "target")) then
			-- add message "can't cooperate"
			DEFAULT_CHAT_FRAME:AddMessage(UnitName("target")..' cannot cooperate.', 1.0, 1.0, 0.5);
		elseif (not CheckInteractDistance("target",2)) then
			-- add message "unit too far away"
			DEFAULT_CHAT_FRAME:AddMessage(UnitName("target")..' is too far away.', 1.0, 1.0, 0.5);
		else
			-- pickup the item and drop it on the target
			PickupContainerItem(bag,slot)
			if ( CursorHasItem() ) then
				DropItemOnUnit("target")
			end
		end
	end
end

--valid params are 
--	s=spellid (number)
--	s=spell name (string)
--	s=list of spells (table of strings)
function XAD:CastSpell(s)
	local spellid
	--check for a spell id
	if type(s)=='number' then 
		spellid = s
	else
		-- ok look up the spell info GetSpellDate accepts a string or table of strings
		spellid, _, _  = self:GetSpellData(s) 
	end
    self:LevelDebug(2, "Found spell %s", spellid)

	--if we didn't find a spell id there is nothing else we can do
	if not spellid then 
		return 
	end
	-- cast the spell
	CastSpell(spellid, "spell")
	--self cast if spell is left on targeting cursor
	if ( SpellIsTargeting() ) then
		SpellTargetUnit("player")
	end
end

--valid input is a spell name or a table of sting holding spell names
--note the highest rank of the first spell found in the table will be returned
function XAD:GetSpellData(s)
	local spellid, spellname
	if type(s)=='table' then
		-- if we have a table of spells then loop through till we find a spell the user doesn't have
		-- and use the last one we successfuly found
		for _, sp in s do
			local sName, sRankText, sId, sFullText, sRank, sIdStart, sIdStop, sRank = SC:GetSpellData(sp)
			self:LevelDebug(2, "Spell data %s %s %s %s %s %s %s %s", sName, sRankText, sId, sFullText, sRank, sIdStart, sIdStop, sRank)
			
			if sId then-- we didn't find a spell so we are done
				--if sIdStop then spellid = sIdStop else spellid = sId end -- use the spellid or it's end point ie highest rank
				spellid = sIdStop or sId -- use the spellid or it's end point ie highest rank
				spellname = sName
				break
			end
		end
		if not spellname then return end
	elseif type(s)=='string' then
		local sName, sRankText, sId, sFullText, sRank, sIdStart, sIdStop, sRank = SC:GetSpellData(s)
		if not sId then return end -- we didn't find a spell so we are done
		--if sIdStop then spellid = sIdStop else spellid = sId end -- use the spellid or it's end point ie highest rank
		spellid = sIdStop or sId -- use the spellid or it's end point ie highest rank
		spellName = sName
	end
	return spellid, spellname, BS:GetSpellIcon(spellname)
end
